#!python
#---------------------------------
# p2-k8s-update-resources: A script to update status of k8s resources in the
# k8s_resources table.
# Store datasource is defined via envars:
# - DB_P2STORE_HOST
# - DB_P2STORE_USER
# - DB_P2STORE_PASSWORD
# - DB_P2STORE_DBNAME
# Workflow:
# 2. get list of namespaces and update them in the table
# 3. for each namespace get the list of deployments and update the table
# 4. get the list of deployments and for each of them check healthy status and update
# record.
import sys
import os
import json
from kubernetes import config, client
from datetime import datetime
from p2.driver.k8s import k8s
from p2_store.k8s_resources import K8sResource
from postgres.client.sql import SQLit

print("Connecting to P2STORE host: {}".format(os.environ.get('DB_P2STORE_HOST',
'localhost')))

# if len(sys.argv) == 1:
#     print("Usage: {} <a context to connect to>".format(sys.argv[0]));
#     exit()

#--------------------------------------------------------------------------------------
# 1. check current context is the one passed in input.
#--------------------------------------------------------------------------------------
if len(sys.argv) < 2:
    print("Please pass the context as first parameter")
    exit()

context = sys.argv[1]
(contexts, current) = config.list_kube_config_contexts()
if context not in [c['name'] for c in contexts]:
    print("Error: context {} not configured".format(context))
    exit()

if os.getenv('KUBERNETES_SERVICE_HOST'):
    config.load_incluster_config()
else:
    config.load_kube_config(context=context)

k8s_api = k8s.Api(context=context)
sql = SQLit()

#--------------------------------------------------------------------------------------
# 2. get list of namespaces and update them in the table
#--------------------------------------------------------------------------------------
namespaces = k8s_api.namespaces()

#--------------------------------------------------------------------------------------
# 3. for each namespace get the list of deployments and update the table
#--------------------------------------------------------------------------------------
for namespace in namespaces:
    record = {
        'context': context,
        'kind': 'namespace',
        'name': namespace['name'],
        'profile': json.dumps(namespace),
        'status': 'active',
        'last_updated': datetime.now()
    }
    namespace_rec = sql.select('k8s_resources',
        context=context,
        kind='namespace',
        name=namespace['name']
        )

    if len(namespace_rec) == 1: 
        sql.update('k8s_resources', 'id', namespace_rec[0]['id'], **record)
        print("Resource {} updated".format(namespace_rec[0]['id']))
    else:
        resource = K8sResource()
        ret = resource.add_resource(record)
        if (ret == 1):
            print("Resource {} inserted".format(namespace['name']))
        else:
            print("Error inserting resource:\n{}".format(namespace))
            continue

    namespace_rec = sql.select('k8s_resources',
        context=context,
        kind='namespace',
        name=namespace['name']
        )
    if len(namespace_rec) == 0:
        print("Something went wrong with resource {} ...".format(namespace['name']))
        continue

    deployments = k8s_api.deployments_for_namespace(namespace['name'])
    for name in deployments.keys():
        deployment_rec = sql.select('k8s_resources', 
                context=context,
                kind='deployment',
                name=name
                )
        healthy = k8s_api.is_deployment_healthy(deployments[name])
        if len(deployment_rec) == 1: 
            sql.update('k8s_resources', 
                'id', deployment_rec[0]['id'], 
                status='healthy' if healthy else 'not healthy',
                profile=json.dumps(deployments[name]),
                last_updated=datetime.now()
                )
        else:
            record = {
                "parent_id": namespace_rec[0]['id'],
                "context": context,
                "kind": "deployment",
                "name": name,
                "profile": json.dumps(deployments[name]),
                "status": 'healthy' if healthy else 'not healthy',
                "last_updated": datetime.now()
            }
            resource = K8sResource()
            ret = resource.add_resource(record)
            if (ret == 1 ):
                print("Resource {} inserted".format(name))
            else:
                print("Error inserting resource:\n{}".format(record))
