"""Main module."""
from flask import Flask, jsonify, request, abort
import p2_store
from p2_store.experiments import Experiment
from p2_store.sessions import Session
from p2_store.logs import Log
from p2_store.k8s_resources import K8sResource
import logging
import json

from logging.handlers import RotatingFileHandler

logging.basicConfig(format='%(asctime)s %(message)s')
logging_obj = logging.getLogger()
logging_obj.setLevel(logging.DEBUG)

handler = RotatingFileHandler(
    'server.log', 
    maxBytes=10000, 
    backupCount=5
    )

# create formatter
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

# add formatter to ch
handler.setFormatter(formatter)

logging_obj.addHandler(handler)
logging_obj.info("Using p2_store version: {}".format(p2_store.__version__))

app = Flask(__name__)
app.config['JSONIFY_PRETTYPRINT_REGULAR'] = True

@app.route('/')
def index():
    return '<br><br><h1> Welcome to Chaos. Interface for accessing and storing Chaos Experiments.</h1>'

@app.route('/store/experiments')
def getE():    
    exp_obj = Experiment()
    return jsonify([i[0] for i in exp_obj.getExperiments()])
  
@app.route('/store/experiment/<filename>')
def getEx(filename):    
    logging_obj.debug("Getting experiment: {}".format(filename))
    exp_obj = Experiment()
    return jsonify(exp_obj.getExperiment(filename))

@app.route('/store/experiment/<filename>', methods=['DELETE'])
def deleteEx(filename):
    logging_obj.debug("Deleting filename: {}".format(filename))
    try:
        exp_obj = Experiment()
        rowcount = exp_obj.deleteExperiment(filename)
        if rowcount == 1:
            return jsonify({"deleted":filename})
        else: 
            return None, 404
    except Exception as err:
        err_str = "ERROR: {}".format(err)
        logging_obj.error(err_str)
        return err_str, 500

@app.route('/store/experiment/<filename>', methods=['POST'])
def createEx(filename):
    try:
        data = request.get_json(force=True)        
        exp_obj = Experiment()
        exp_obj.postExperiment(filename, data)
        return jsonify({"filename":filename, "data":data})
    except Exception as err:
        logging_obj.error("ERROR: {}".format(err))
        return abort(500)

@app.route('/store/experiment/<filename>', methods=['PUT'])
def updateEx(filename):
    try:
        data = request.get_json(force=True)        
        exp_obj = Experiment()
        count = exp_obj.updateExperiment(filename, data)
        if (count == 1):
            return jsonify(exp_obj.getExperiment(filename))
        else:
            logging_obj.error("[updateEx]: experiment {} not found".format(filename))
            return abort(400)
    except Exception as err:
        logging_obj.error("ERROR: {}".format(err))
        return abort(500)


@app.route('/store/exp_session/<exp_id>', methods=['POST'])
def create_session(exp_id):
    try:
        data = request.get_json(force=True)        
        data['exp_id'] = exp_id
        session = Session()
        session.create_session(data)
        return jsonify(data), 201
    except Exception as err:
        logging_obj.error("POST Session Error: {}".format(err))
        return abort(500)

@app.route('/store/exp_sessions', methods=['GET'])
def get_paged_sessions():
    """ query params:
        - size: number of returned records, not mandatory, defaults to 10
        - offset: index of next record to retrieve, not mandatory
    """
    try:
        args = request.args
        session = Session()
        size=None
        offset=None
        if "size" in args:
            size=args["size"]
        if "offset" in args:
            offset=args["offset"]
        records = session.get_sessions_paged(size=size, offset=offset)
        return jsonify(records), 200

    except Exception as err:
        logging_obj.error("GET paged sessions Error: {}".format(err))
        return abort(500)

@app.route('/store/exp_session', methods=['GET'])
def get_session():
    try:
        filter = request.args.to_dict()
        logging_obj.info("[get_session]: filter: {}".format(filter))
        session = Session()
        records = session.get_sessions_as_dict( filter )
        logging_obj.info("Records: {}".format(records))
        return jsonify(records), 200
    except Exception as err:
        logging_obj.error("GET session Error: {}".format(err))
        return abort(500)

@app.route('/store/exp_session/<exp_id>', methods=['PUT'])
def update_session(exp_id):
    try:
        attrs = request.get_json(force=True)        
        logging_obj.debug("Updating session {}".format(exp_id))
        logging_obj.debug("Attrs: {}".format(attrs))

        session = Session()
        ret = session.update_session(exp_id, attrs)
        if (ret == 1):
            return jsonify(data), 200
        else:
            return jsonify({exp_id:"Error updating"}), 400
    except Exception as err:
        logging_obj.error("UPDATE session Error: {}".format(err))
        return abort(500)

@app.route('/store/exp_log/<exp_id>', methods=['POST'])
def create_log(exp_id):
    try:

        data = request.get_json(force=True)        
        logging_obj.debug("Creating log record: {}".format(log_rec))
        log_obj = Log()
        if (log_obj.create_log(exp_id, data) == 1):
            logging_obj.info("Log created")
            return data
        else:
            logging_obj.error("Error creating log")
            return "Error creating log", 404
    except Exception as err:
        logging_obj.error("POST Log Error: {}".format(err))
        return abort(500)

@app.route('/store/experiment/log/<exp_id>/<log_type>', methods=['GET'])
def get_log(exp_id, log_type):
    logging_obj.debug("Getting log type {} for exp_id {}".format(log_type, exp_id))
    try:
        log_obj = Log()
        ret = log_obj.get_logs({'exp_id': exp_id, 'log_type': log_type})
        return jsonify(ret)
    except Exception as err:
        logging_obj.error("GET log Error: {}".format(err))
        return abort(500)


@app.route('/store/experiment/log/<exp_id>', methods=['POST'])
def add_log(exp_id):
    logging_obj.debug("Adding a log for exp_id {}".format(exp_id))
    try:
        attrs = request.get_json(force=True)        
        logging_obj.debug("Attrs: {}".format(attrs))
        log_obj = Log()
        log_obj.create_log(exp_id, attrs)
        return jsonify({exp_id:"Log added"}), 201
    except Exception as err:
        logging_obj.error("UPDATE log Error: {}".format(err))
        return abort(500)

@app.route('/store/k8s_resources', methods=['GET'])
def get_k8s_resources():
    """ query params:
        - kind: kind of resource to get, mandatory 
        - size: number of returned records, not mandatory, defaults to 1000
        - offset: index of next record to retrieve, not mandatory
    """
    try:
        resource = K8sResource()
        size=None
        offset=None

        args = request.args
        if 'kind' not in args:
            logging_obj.error("'kind' query parameter has to be passed to the request")
            return abort(500)
        kind = args['kind']

        if "size" in args:
            size=args["size"]
        if "offset" in args:
            offset=args["offset"]
        records = resource.get_resources_of_kind(kind, size=size, offset=offset)
        return jsonify(records), 200

    except Exception as err:
        logging_obj.error("GET k8s resources Error: {}".format(err))
        return abort(403)

@app.route('/store/k8s_resource', methods=['POST'])
def add_resource():
    try:
        data = request.get_json(force=True)        
        resource = K8sResource()
        resource.add_resource(data)
        return jsonify('{responsedata : %s was posted}'%(data['name'])), 201
    except Exception as err:
        logging_obj.error("POST K8s Resource Error: {}".format(err))
        return abort(400)

@app.route('/store/list_routes', methods=['GET'])
def list_routes():
    logging_obj.debug("Printing list of routes")
    return jsonify(['%s' % rule for rule in app.url_map.iter_rules()])

if __name__ == '__main__':
    app.run(use_evalex=True, use_debugger=True, host='0.0.0.0', port=5000)
